/*
  Empfänger
  =========

  TODO:
  Offset für negative Temperaturen.
*/
#include <T2WhisperNode.h>

/* You need to configure the Whisper Node Version */
#define T2_WPN_BOARD T2_WPN_VER_RF69
//#define T2_WPN_BOARD T2_WPN_VER_LORA

#define RADIO_FREQUENCY 868.0
#define RADIO_TX_POWER 13
#define RADIO_ENCRYPTION_KEY { 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F } // Only used by RF69

#include <RH_RF69.h>
RH_RF69 myRadio;

// LED
T2Led myLedBlue(T2_WPN_LED_1);
T2Led myLedYellow(T2_WPN_LED_2);

// Radio
uint8_t radioBuf[(T2_MESSAGE_HEADERS_LEN + T2_MESSAGE_MAX_DATA_LEN)];

// T2 Message
T2Message myMsg;
#define nodeAddr 0x88
#define baseAddr 0x0B

// T2 Data Buffer
T2DataBuffer myDataBuffer;

// Stand-alone Task
void taskPrintFunction(T2Task * Task); // Implemented at the bottom of this file
T2Task myTaskPrint(&taskPrintFunction);

// Global Variables to hold the Received data
uint16_t batVoltage = 0;
uint16_t supVoltage = 0;
uint16_t temB280    = 0;
uint16_t humB280    = 0;

//==============================================================================
void runReceiver()
{
  /* RFM69 Receive */
  uint8_t radioBufLen = sizeof(radioBuf);
  if (myRadio.recv(radioBuf, &radioBufLen))
  {
    myMsg.setSerializedMessage(radioBuf, radioBufLen);
    // Uncomment bellow to print every received message, just be careful as
    // delays here can cause messages to be lost.
    //myMsg.printMessage();

    // We can perform some message filtering based on it's headers
    if (myMsg.idx == 0x06 && myMsg.src == nodeAddr && myMsg.dst == baseAddr)
    {

      // Lets blink something
      myLedBlue.setBlink(10, 0, 1); // LED will lit for 10ms only once, so the interval doesn't matter

      switch (myMsg.sdx)
      {
        case 0x64: // Battery Voltage
          // Concatenate 2 bytes into a uint16_t variable
          batVoltage = myMsg.data[2] << 8;
          batVoltage |= myMsg.data[3];
          break;

        case 0x65: // Power Supply Voltage
          // Concatenate 2 bytes into a uint16_t variable
          supVoltage = myMsg.data[2] << 8;
          supVoltage |= myMsg.data[3];
          break;

        case 0x66: // Power Supply Voltage
          // Concatenate 2 bytes into a uint16_t variable
          temB280 = myMsg.data[2] << 8;
          temB280 |= myMsg.data[3];
          break;

        case 0x67: // Power Supply Voltage
          // Concatenate 2 bytes into a uint16_t variable
          humB280 = myMsg.data[2] << 8;
          humB280 |= myMsg.data[3];
          break;

        default: // Can define an operation for everything else
          // Do something
          Serial.println(F("Unexpected message received: "));
          myMsg.printMessage();
          break;
      }
    }
  }
}
//==============================================================================
void taskPrintFunction(T2Task * Task)
{
  // We simply print the last data we got
  Serial.print(F("Bat: ")); Serial.print(batVoltage); Serial.println(F(" mV"));
  Serial.print(F("Pow: ")); Serial.print(supVoltage); Serial.println(F(" mV"));
  Serial.print(F("Tmp: ")); Serial.print(temB280);    Serial.println(F(" °C"));
  Serial.print(F("Hum: ")); Serial.print(humB280);    Serial.println(F(" %rH"));
  Serial.println("=============================");
}
//==============================================================================
void setup()
{
  // Serial
  Serial.begin(115200);
  Serial.println(F("Voltage - Base"));

  // Radio
  myRadio.init();
  myRadio.setFrequency(RADIO_FREQUENCY);
  myRadio.setTxPower(RADIO_TX_POWER);
#if T2_WPN_BOARD == T2_WPN_VER_RF69
  uint8_t myRadioEncryptionKey[] = RADIO_ENCRYPTION_KEY;
  myRadio.setEncryptionKey(myRadioEncryptionKey);
#endif

  // Stand-alone Task
  myTaskPrint.runInterval(5000);   // Task will tun every 1000 milliseconds
  myTaskPrint.runCount(0);         // Task will run forever
  myTaskPrint.status(T2TASK_STATUS_SCHEDULED);

  // LED Example
  myLedYellow.setBlink(20, 3000, -1);   // LED will lit for 20ms every 3000ms, forever
}
//==============================================================================
void loop()
{
  // Stand-alone Task - We run it as a non-blocking Task to print the readings
  myTaskPrint.run();

  // LED Example
  myLedBlue.run();
  myLedYellow.run();

  // Run the Receiving function
  runReceiver();
}
//==============================================================================
